/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.RhinoTests.Reporter;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;



/**
 * Generator for HTML pages which contains graphs with test results.
 *
 * @author Pavel Tisnovsky
 */
public class GraphPagesGenerator {

    public static void generate(Map<String, Map<String, List<String>>> testResults, CommandLineParameters params) {
        createHtmlPageFromTemplate(testResults, params, 10);
        createHtmlPageFromTemplate(testResults, params, 20);
        createHtmlPageFromTemplate(testResults, params, 30);
        createHtmlPageFromTemplate(testResults, params, -1);
    }

    private static void createHtmlPageFromTemplate(Map<String, Map<String, List<String>>> testResults, CommandLineParameters params, int testCount) {
        List<String> template = FileUtils.readTextFile(params.getTemplateDir() + "/graph.html");
        List<String> out = new LinkedList<String>();
        List<TestResult> testResultList = processTestResultList(testResults, testCount);

        // iterate through whole template
        for (String templateLine : template) {
            // replace text in template where needed
            if (templateLine.contains("${TEST_COUNT}")) {
                String testCountMessage = testCount == -1 ? "all" : "the last " + testCount;
                templateLine = templateLine.replace("${TEST_COUNT}", testCountMessage);
                out.add(templateLine);
            }
            else if (templateLine.contains("${GRAPH_DATA_PASSED}")) {
                renderGraphDataPassed(out, testResultList);
            }
            else if (templateLine.contains("${GRAPH_DATA_FAILED}")) {
                renderGraphDataFailed(out, testResultList);
            }
            else if (templateLine.contains("${GRAPH_DATA_ERROR}")) {
                renderGraphDataError(out, testResultList);
            }
            else if (templateLine.contains("${TABLE_DATA}")) {
                renderTableData(out, testResults, testCount);
            }
            else {
                out.add(templateLine);
            }
        }
        // write list of string to a file with given name
        String outName = testCount == -1 ? "graph_all" : "graph_" + testCount;
        FileUtils.writeTextFile(params.getReportDir() + "/" + outName + ".html", out);
    }

    private static String oneRecord(int i, int cnt) {
        return "[" + (i+1) + ", " + cnt + "],";
    }

    private static void renderGraphDataPassed(List<String> out, List<TestResult> testResultList) {
        for (int i = 0; i < testResultList.size(); i++) {
            out.add(oneRecord(i, testResultList.get(i).getPassed()));
        }
    }

    private static void renderGraphDataFailed(List<String> out, List<TestResult> testResultList) {
        for (int i = 0; i < testResultList.size(); i++) {
            out.add(oneRecord(i, testResultList.get(i).getFailed()));
        }
    }

    private static void renderGraphDataError(List<String> out, List<TestResult> testResultList) {
        for (int i = 0; i < testResultList.size(); i++) {
            out.add(oneRecord(i, testResultList.get(i).getError()));
        }
    }

	private static List<TestResult> processTestResultList(
			Map<String, Map<String, List<String>>> testResults, int testCount) {
        List<TestResult> testResultList = new LinkedList<TestResult>();
        int threshold = testCount == -1 ? 0 : testResults.size() - testCount;
        int index = 0;
        for (Map.Entry<String, Map<String, List<String>>> oneResult : testResults.entrySet()) {
            if (index >= threshold) {
                TestResult testResult = TestResult.readSummary(oneResult.getValue());
                testResultList.add(testResult);
            }
            index++;
        }
        return testResultList;
    }

	private static void renderTableData(List<String> out,
			Map<String, Map<String, List<String>>> testResults, int testCount) {
        int threshold = testCount == -1 ? 0 : testResults.size() - testCount;
        int index = 0;
        int rowIndex = 1;
        for (Map.Entry<String, Map<String, List<String>>> oneResult : testResults.entrySet()) {
            if (index >= threshold) {
                String date = oneResult.getKey();
                TestResult testResult = TestResult.readSummary(oneResult.getValue());
                out.add("<tr><td>" + rowIndex + "</td><td><a href='log_" + date + ".html'>&nbsp;" + date + "&nbsp;</a></td>");
                out.add("<td style='color:#006000;text-align:right;'>" + testResult.getPassed() + "</td>");
                out.add("<td style='color:#800000;text-align:right;'>" + testResult.getFailed() + "</td>");
                out.add("<td style='color:#000080;text-align:right;'>" + testResult.getError() + "</td>");
                out.add("</tr>");
            }
            index++;
            rowIndex++;
        }
    }

}
